@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
+   Copyright (C) 2009-2012, 2014-2017 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <iostream>
 #include <cstdlib>
@@ -77,8 +75,8 @@ main (int argc, char **argv)
   /* Debugging trace. */
   if (debug)
     {
-      Debug debug = mbox.get_debug ();
-      debug.set_level (MU_DEBUG_LEVEL_UPTO (MU_DEBUG_PROT));
+      mu_debug_set_category_level (MU_DEBCAT_MAILBOX, 
+				   MU_DEBUG_LEVEL_UPTO (MU_DEBUG_PROT));
     }
 
   /* Open the mailbox for reading only. */
@@ -187,14 +185,12 @@ message_display_parts (Message& msg, int indent)
 	  Stream stream = body.get_stream ();
 
 	  FilterStream filter (stream, encoding, 0, 0);
-          int offset = 0;
 	  char buf[2048];
 
-          while (filter.readline (buf, sizeof (buf), offset) == 0 &&
+          while (filter.readline (buf, sizeof (buf)) == 0 &&
 		 filter.get_read_count ())
             {
 	      cout << setw (indent) << setfill (' ') << buf;
-              offset += filter.get_read_count ();
             }
         }
       else
