@@ -1,5 +1,5 @@
 #  GNU Mailutils -- a suite of utilities for electronic mail
-#  Copyright (C) 2009, 2010, 2011 Free Software Foundation, Inc.
+#  Copyright (C) 2009-2012, 2014-2017 Free Software Foundation, Inc.
 #
 #  GNU Mailutils is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -23,22 +23,13 @@ if len (sys.argv) != 3:
     sys.exit (0)
 
 sti = stream.StdioStream (stream.MU_STDIN_FD)
-sti.open ()
-
 cvt = filter.FilterIconvStream (sti, sys.argv[1], sys.argv[2])
-cvt.open ()
-
 out = stream.StdioStream (stream.MU_STDOUT_FD, 0)
-out.open ()
 
 total = 0
 while True:
-    buf = cvt.read (total)
+    buf = cvt.read ()
     out.write (buf)
     total += cvt.read_count
     if not cvt.read_count:
         break
-
-out.flush ()
-out.close ()
-sti.close ()
