@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2007-2012, 2014-2016 Free Software
+   Copyright (C) 1999, 2001, 2007-2012, 2014-2017 Free Software
    Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -36,6 +36,7 @@ imap4d_delete (struct imap4d_session *session,
   const char *msg = "Completed";
   char *name;
   mu_mailbox_t tmpbox;
+  mu_record_t record;
   
   if (imap4d_tokbuf_argc (tok) != 3)
     return io_completion_response (command, RESP_BAD, "Invalid arguments");
@@ -43,17 +44,17 @@ imap4d_delete (struct imap4d_session *session,
   if (!name || *name == '\0')
     return io_completion_response (command, RESP_BAD, "Too few arguments");
 
-  /* It is an error to attempt to delele "INBOX or a mailbox
-     name that dos not exists.  */
+  /* It is an error to attempt to delele "INBOX" or a mailbox
+     name that does not exist.  */
   if (mu_c_strcasecmp (name, "INBOX") == 0)
-    return io_completion_response (command, RESP_NO, "Already exist");
+    return io_completion_response (command, RESP_NO, "Cannot remove");
 
- /* Allocates memory.  */
-  name = namespace_getfullpath (name, NULL);
+  /* Allocates memory.  */
+  name = namespace_get_name (name, &record, NULL);
   if (!name)
     return io_completion_response (command, RESP_NO, "Cannot remove");
 
-  rc = mu_mailbox_create (&tmpbox, name);
+  rc = mu_mailbox_create_from_record (&tmpbox, record, name);
   if (rc == 0)
     {
       imap4d_enter_critical ();
@@ -69,7 +70,8 @@ imap4d_delete (struct imap4d_session *session,
       if (rc)
 	mu_diag_funcall (MU_DIAG_ERROR, "remove", name, errno);
     }
-
+  free (name);
+  
   if (rc)
     {
       rc = RESP_NO;
