@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail -*- c -*-
-   Copyright (C) 1999, 2000, 2005, 2007, 2008, 2010, 2011 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999-2000, 2005, 2007-2008, 2010-2012, 2014-2017 Free
+   Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -81,7 +81,7 @@ void mu_debug_log_cont (const char *fmt, ...) MU_PRINTFLIKE(1,2);
 void mu_debug_log_end (const char *fmt, ...) MU_PRINTFLIKE(1,2);
 void mu_debug_log_nl (void);
 
-  int mu_debug_get_iterator (mu_iterator_t *piterator, int skipunset);
+int mu_debug_get_iterator (mu_iterator_t *piterator, int skipunset);
 
   
 #define MU_ASSERT(expr)						\
