@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2005-2007, 2010-2012, 2014-2016 Free
+   Copyright (C) 1999-2001, 2005-2007, 2010-2012, 2014-2017 Free
    Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -211,10 +211,19 @@ extern int mu_message_set_bodystructure (mu_message_t msg,
       void *owner);
   
 /* misc functions */
+extern int mu_attachment_create (mu_message_t *newmsg,
+				 const char *content_type,
+				 const char *encoding,
+				 const char *name, const char *filename);
+extern int mu_attachment_copy_from_stream (mu_message_t att,
+					   mu_stream_t stream);
+extern int mu_attachment_copy_from_file (mu_message_t att,
+					 char const *filename);
 extern int mu_message_create_attachment (const char *content_type,
 					 const char *encoding,
 					 const char *filename,
 					 mu_message_t *newmsg);
+  
 extern int mu_message_save_attachment (mu_message_t msg,
 				       const char *filename,
 				       mu_mime_io_buffer_t buf);
