@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009, 2011-2012 Free Software Foundation, Inc.
+   Copyright (C) 2009, 2011-2012, 2014-2017 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -73,6 +74,11 @@ enum mu_buffer_type
 #define MU_IOCTL_FD               9 /* File descriptor manipulation */
 #define MU_IOCTL_SYSLOGSTREAM    10 /* Syslog stream (see below) */
 #define MU_IOCTL_FILTER          11 /* Filter streams (see below) */
+#define MU_IOCTL_TOPSTREAM       12 /* Same as MU_IOCTL_SUBSTREAM, but
+				       always returns the topmost substream.
+				    */
+#define MU_IOCTL_TLSSTREAM       13 /* TLS stream */
+#define MU_IOCTL_WORDWRAPSTREAM  14 /* Word-wrapper stream */
   
   /* Opcodes common for various families */
 #define MU_IOCTL_OP_GET 0
@@ -88,7 +94,7 @@ enum mu_buffer_type
      If pat==NULL, any reads from that stream will return EOF. */
 #define MU_IOCTL_NULLSTREAM_SET_PATTERN 0
   /* Set pattern class for reads:  Argument int *pclass (a class mask
-     from mailutils/cctype.h */
+     from mailutils/cctype.h) */
 #define MU_IOCTL_NULLSTREAM_SET_PATCLASS 1
   /* Limit stream size.  Argument: mu_off_t *psize; */
 #define MU_IOCTL_NULLSTREAM_SETSIZE 2
@@ -188,12 +194,31 @@ enum mu_buffer_type
   */
 #define MU_IOCTL_FILTER_GET_DISABLED 0
 #define MU_IOCTL_FILTER_SET_DISABLED 1  
+
+  /* TLS transport streams */
+  /* Get cipher info.
+     Arg: mu_property_t *
+     On success, the following keys are defined: "protocol", "cipher", "mac"
+  */
+#define MU_IOCTL_TLS_GET_CIPHER_INFO 0
   
 #define MU_TRANSPORT_INPUT  0
 #define MU_TRANSPORT_OUTPUT 1
 #define MU_TRANSPORT_VALID_TYPE(n) \
   ((n) == MU_TRANSPORT_INPUT || (n) == MU_TRANSPORT_OUTPUT)
 
+/* Word wrapper streams */
+/* Get left margin. */
+#define MU_IOCTL_WORDWRAP_GET_MARGIN      0
+/* Set left margin */  
+#define MU_IOCTL_WORDWRAP_SET_MARGIN      1
+/* Shift left margin relative to current position */ 
+#define MU_IOCTL_WORDWRAP_MOVE_MARGIN     2
+/* Set left margin for the next line */
+#define MU_IOCTL_WORDWRAP_SET_NEXT_MARGIN 3
+/* Get current column */
+#define MU_IOCTL_WORDWRAP_GET_COLUMN      4
+
 struct mu_nullstream_pattern
 {
   char *pattern;
@@ -348,6 +373,9 @@ int mu_rdcache_stream_create (mu_stream_t *pstream, mu_stream_t transport,
 			      int flags);
 
 int mu_nullstream_create (mu_stream_t *pref, int flags);
+
+int mu_wordwrap_stream_create (mu_stream_t *pstream, mu_stream_t transport,
+			       size_t left_margin, size_t right_margin);
   
 #ifdef __cplusplus
 }
