@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 2010-2012, 2014-2017 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
@@ -40,7 +40,7 @@ struct _mu_tls_stream
 {
   struct _mu_stream stream;
   enum _mu_tls_stream_state state;
-  gnutls_session session;
+  gnutls_session_t session;
   int tls_err;
   mu_stream_t transport[2];
 };
