@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2007 Free Software Foundation, Inc.
+   Copyright (C) 2007-2008, 2010-2012, 2014-2017 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,23 +13,30 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see 
+   <http://www.gnu.org/licenses/>. */
 
 #ifndef _MAILUTILS_VERSION_H
 #define _MAILUTILS_VERSION_H
 
-#include <stdio.h>  
+#include <mailutils/types.h>  
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-extern char *mu_license_text;
+struct mu_conf_option
+{
+  char *name;
+  char *descr;
+};
+  
 extern void mu_print_options (void);
-extern void mu_fprint_options (FILE *fp);
-extern const char *mu_check_option (char *name);
+extern void mu_format_options (mu_stream_t, int verbose);
+extern void mu_format_conf_option (mu_stream_t,
+                                   const struct mu_conf_option *opt,
+				   int verbose);
+extern const struct mu_conf_option *mu_check_option (char *name);
 
 #ifdef __cplusplus
 }
