@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2005, 2007, 2009-2012 Free Software Foundation, Inc.
+   Copyright (C) 2005, 2007, 2009-2012, 2014-2017 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,11 +19,7 @@
 # include <config.h>
 #endif
 #include <mailutils/mailutils.h>
-#include <xalloc.h>
 #include <fnmatch.h>
-#define obstack_chunk_alloc malloc
-#define obstack_chunk_free free
-#include <obstack.h>  
 #include <sys/wait.h>
 #include <ctype.h>
 
@@ -87,7 +84,7 @@ mime_context_fill (struct mime_context *ctx, const char *file,
 
   ctx->content_type = ws.ws_wordv[0];
   ws.ws_wordv[0] = NULL;
-  ctx->temp_file = file ? strdup (file) : NULL; 
+  ctx->temp_file = file ? mu_strdup (file) : NULL; 
   ctx->unlink_temp_file = 0;
 
   if (interactive)
@@ -242,7 +239,7 @@ mime_context_get_temp_file (struct mime_context *ctx, char **ptr)
 }
 
 
-static struct obstack expand_stack;
+static mu_opool_t expand_pool;
 
 static int
 expand_string (struct mime_context *ct, char **pstr)
@@ -259,14 +256,14 @@ expand_string (struct mime_context *ct, char **pstr)
 	      {
 	      case 's':
 		mime_context_get_temp_file (ct, &s);
-		obstack_grow (&expand_stack, s, strlen (s));
+		mu_opool_appendz (expand_pool, s);
 		rc = 1;
 		p += 2;
 		break;
 		
 	      case 't':
 		mime_context_get_content_type (ct, &s);
-		obstack_grow (&expand_stack, s, strlen (s));
+		mu_opool_appendz (expand_pool, s);
 		p += 2;
 		break;
 		
@@ -282,7 +279,7 @@ expand_string (struct mime_context *ct, char **pstr)
 		  if (mime_context_get_content_type_value (ct,
 							   q, p-q,
 							   &s, &n) == 0)
-		    obstack_grow (&expand_stack, s, n);
+		    mu_opool_append (expand_pool, s, n);
 		  if (*p)
 		    p++;
 		  break;
@@ -294,19 +291,19 @@ expand_string (struct mime_context *ct, char **pstr)
 		break;
 		
 	      default:
-		obstack_1grow (&expand_stack, p[0]);
+		mu_opool_append_char (expand_pool, p[0]);
 	      }
 	    break;
 
 	case '\\':
 	  if (p[1])
 	    {
-	      obstack_1grow (&expand_stack, p[1]);
+	      mu_opool_append_char (expand_pool, p[1]);
 	      p += 2;
 	    }
 	  else
 	    {
-	      obstack_1grow (&expand_stack, p[0]);
+	      mu_opool_append_char (expand_pool, p[0]);
 	      p++;
 	    }
 	  break;
@@ -314,23 +311,23 @@ expand_string (struct mime_context *ct, char **pstr)
 	case '"':
 	  if (p[1] == p[0])
 	    {
-	      obstack_1grow (&expand_stack, '%');
+	      mu_opool_append_char (expand_pool, '%');
 	      p++;
 	    }
 	  else
 	    {
-	      obstack_1grow (&expand_stack, p[0]);
+	      mu_opool_append_char (expand_pool, p[0]);
 	      p++;
 	    }
 	  break;
 
 	default:
-	  obstack_1grow (&expand_stack, p[0]);
+	  mu_opool_append_char (expand_pool, p[0]);
 	  p++;
 	}
     }
-  obstack_1grow (&expand_stack, 0);
-  *pstr = obstack_finish (&expand_stack);
+  mu_opool_append_char (expand_pool, 0);
+  *pstr = mu_opool_finish (expand_pool, NULL);
   return rc;
 }
 
@@ -426,13 +423,11 @@ create_filter (char *cmd, int outfd, int *infd)
     {
       /* Child process */
       struct mu_wordsplit ws;
-      int argc;
       char **argv;
       
       if (need_shell_p (cmd))
 	{
 	  char *x_argv[4];
-	  argc = 3;
 	  argv = x_argv;
 	  argv[0] = getenv ("SHELL");
 	  argv[1] = "-c";
@@ -447,7 +442,6 @@ create_filter (char *cmd, int outfd, int *infd)
 			mu_wordsplit_strerror (&ws));
 	      _exit (127);
 	    }
-	  argc = ws.ws_wordc;
 	  argv = ws.ws_wordv;
 	}      
       /* Create input channel: */
@@ -515,8 +509,8 @@ run_test (mu_mailcap_entry_t entry, struct mime_context *ctx)
       struct mu_wordsplit ws;
       char *str;
 
-      obstack_blank (&expand_stack, size + 1);
-      str = obstack_finish (&expand_stack);
+      mu_opool_alloc (expand_pool, size + 1);
+      str = mu_opool_finish (expand_pool, NULL);
       mu_mailcap_entry_get_test (entry, str, size + 1, NULL);
 
       expand_string (ctx, &str);
@@ -544,7 +538,7 @@ run_mailcap (mu_mailcap_entry_t entry, struct mime_context *ctx)
   int fd;                
   int *pfd = NULL;      
   int outfd = -1;       
-  pid_t pid, pager_pid;
+  pid_t pid;
   
   if (ctx->debug_level > 1)
     dump_mailcap_entry (entry);
@@ -557,8 +551,8 @@ run_mailcap (mu_mailcap_entry_t entry, struct mime_context *ctx)
       if (mu_mailcap_entry_get_viewcommand (entry, NULL, 0, &size))
 	return 1;
       size++;
-      obstack_blank (&expand_stack, size);
-      view_command = obstack_finish (&expand_stack);
+      mu_opool_alloc (expand_pool, size);
+      view_command = mu_opool_finish (expand_pool, NULL);
       mu_mailcap_entry_get_viewcommand (entry, view_command, size, NULL);
     }
   else
@@ -566,8 +560,8 @@ run_mailcap (mu_mailcap_entry_t entry, struct mime_context *ctx)
       if (mu_mailcap_entry_get_value (entry, "print", NULL, 0, &size))
 	return 1;
       size++;
-      obstack_blank (&expand_stack, size);
-      view_command = obstack_finish (&expand_stack);
+      mu_opool_alloc (expand_pool, size);
+      view_command = mu_opool_finish (expand_pool, NULL);
       mu_mailcap_entry_get_value (entry, "print", view_command, size, NULL);
     }
 
@@ -585,7 +579,7 @@ run_mailcap (mu_mailcap_entry_t entry, struct mime_context *ctx)
   flag = 0;
   if (interactive_p (ctx)
       && mu_mailcap_entry_copiousoutput (entry, &flag) == 0 && flag)
-    pager_pid = create_filter (get_pager (), -1, &outfd);
+    create_filter (get_pager (), -1, &outfd);
   
   pid = create_filter (view_command, outfd, pfd);
   if (pid > 0)
@@ -692,7 +686,7 @@ display_stream_mailcap (const char *ident, mu_stream_t stream, mu_header_t hdr,
       mailcap_path = mailcap_path_tmp;
     }
   
-  obstack_init (&expand_stack);
+  mu_opool_create (&expand_pool, MU_OPOOL_ENOMEMABRT);
 
   ws.ws_delim = ":";
   if (mu_wordsplit (mailcap_path, &ws,
@@ -713,7 +707,7 @@ display_stream_mailcap (const char *ident, mu_stream_t stream, mu_header_t hdr,
 	}
       mu_wordsplit_free (&ws);
     }
-  obstack_free (&expand_stack, NULL);
+  mu_opool_destroy (&expand_pool);
   free (mailcap_path_tmp);
   mime_context_release (&ctx);
   return rc;
