@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2006-2007, 2009-2012, 2014-2016 Free Software
+   Copyright (C) 2006-2007, 2009-2012, 2014-2017 Free Software
    Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -74,17 +74,17 @@
 #include <stdlib.h>
 
 static int
-moderator_filter_message (mu_sieve_machine_t mach, mu_list_t tags,
+moderator_filter_message (mu_sieve_machine_t mach, 
 			  mu_message_t msg, int *pdiscard)
 {
   int rc;
   mu_sieve_machine_t newmach;
   mu_attribute_t attr;
-  mu_sieve_value_t *arg;
+  char *arg;
   
-  if (mu_sieve_tag_lookup (tags, "source", &arg))
+  if (mu_sieve_get_tag (mach, "source", SVT_STRING, &arg))
     {
-      rc = mu_sieve_machine_inherit (mach, &newmach);
+      rc = mu_sieve_machine_clone (mach, &newmach);
       if (rc)
 	{
 	  mu_sieve_error (mach, _("cannot initialize sieve machine: %s"),
@@ -97,15 +97,15 @@ moderator_filter_message (mu_sieve_machine_t mach, mu_list_t tags,
 	   ...
       */
       
-      rc = mu_sieve_compile (newmach, arg->v.string);
+      rc = mu_sieve_compile (newmach, arg);
       if (rc)
-	mu_sieve_error (mach, _("cannot compile source `%s'"), arg->v.string);
+	mu_sieve_error (mach, _("cannot compile source `%s'"), arg);
     }
-  else if (mu_sieve_tag_lookup (tags, "program", &arg))
+  else if (mu_sieve_get_tag (mach, "program", SVT_STRING, &arg))
     {
       struct mu_locus locus;
       
-      rc = mu_sieve_machine_inherit (mach, &newmach);
+      rc = mu_sieve_machine_clone (mach, &newmach);
       if (rc)
 	{
 	  mu_sieve_error (mach, _("cannot initialize sieve machine: %s"),
@@ -114,7 +114,7 @@ moderator_filter_message (mu_sieve_machine_t mach, mu_list_t tags,
 	}
       mu_sieve_get_locus (mach, &locus);
       rc = mu_sieve_compile_buffer (newmach,
-				    arg->v.string, strlen (arg->v.string),
+				    arg, strlen (arg),
 				    locus.mu_file, locus.mu_line);
       if (rc)
 	mu_sieve_error (mach, _("cannot compile subprogram"));
@@ -265,7 +265,7 @@ moderator_message_get_part (mu_sieve_machine_t mach,
 }
 
 static int
-moderator_action (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
+moderator_action (mu_sieve_machine_t mach)
 {
   mu_message_t msg, orig;
   int rc;
@@ -273,12 +273,6 @@ moderator_action (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
   int discard = 0;
   int ismime;
   
-  if (mu_sieve_get_debug_level (mach) & MU_SIEVE_DEBUG_TRACE)
-    {
-      mu_sieve_debug (mach, "moderator_test %lu",
-		      (unsigned long) mu_sieve_get_message_num (mach));
-    }
-
   msg = mu_sieve_get_message (mach);
   mu_message_is_multipart (msg, &ismime);
 
@@ -300,7 +294,7 @@ moderator_action (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
   if ((rc = moderator_message_get_part (mach, msg, 2, &orig)))
     mu_sieve_abort (mach);
 
-  rc = moderator_filter_message (mach, tags, orig, &discard);
+  rc = moderator_filter_message (mach, orig, &discard);
   mu_message_unref (orig);
   if (rc)
     mu_sieve_abort (mach);
@@ -309,7 +303,6 @@ moderator_action (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
     {
       mu_message_t request;
       char *from = NULL;
-      mu_sieve_value_t *arg;
       
       if ((rc = moderator_message_get_part (mach, msg, 3, &request)))
 	{
@@ -318,14 +311,13 @@ moderator_action (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
 	  mu_sieve_abort (mach);
 	}
 
-      if (mu_sieve_tag_lookup (tags, "address", &arg))
-	from = arg->v.string;
+      mu_sieve_get_tag (mach, "address", SVT_STRING, &from);
       
       if (moderator_discard_message (mach, request, from))
 	discard = 0;
       else
 	{
-	  if (!mu_sieve_tag_lookup (tags, "keep", NULL))
+	  if (!mu_sieve_get_tag (mach, "keep", SVT_VOID, NULL))
 	    {
 	      mu_attribute_t attr = 0;
 
@@ -371,8 +363,9 @@ static mu_sieve_tag_group_t moderator_tag_groups[] = {
 int
 SIEVE_EXPORT(moderator,init) (mu_sieve_machine_t mach)
 {
-  return mu_sieve_register_action (mach, "moderator", moderator_action,
-				   moderator_req_args,
-				   moderator_tag_groups, 1);
+  mu_sieve_register_action (mach, "moderator", moderator_action,
+			    moderator_req_args,
+			    moderator_tag_groups, 1);
+  return 0;
 }
    
