@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2005, 2007-2012, 2014-2016 Free Software
+   Copyright (C) 1999-2005, 2007-2012, 2014-2017 Free Software
    Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -67,10 +67,16 @@ retrieve_address (void *item, void *data, size_t idx, char **pval)
       rc = mu_header_aget_value ((mu_header_t)ap->data, (char*)item, &val);
       if (rc)
 	return rc;
+      if (mu_str_skip_class (val, MU_CTYPE_BLANK)[0] == 0)
+	return MU_ERR_NOENT;
       rc = mu_address_create (&ap->addr, val);
       free (val);
       if (rc)
-	return rc;
+        {
+          if (rc == MU_ERR_EMPTY_ADDRESS)
+            rc = MU_ERR_NOENT;
+          return rc;
+        }
     }
 
   rc = ap->aget (ap->addr, idx+1, pval);
@@ -150,11 +156,7 @@ retrieve_header (void *item, void *data, size_t idx, char **pval)
 	{
 	  int i = hc->index++;
 	  if (mu_c_strcasecmp (hname, (char*)item) == 0)
-	    {
-	      if (mu_header_aget_field_value_unfold (hc->header, i, pval))
-		return -1;
-	      return 0;
-	    }
+	    return mu_header_aget_field_value_unfold (hc->header, i, pval);
 	}
 
       hc->header = NULL;
