@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 1999-2005, 2007-2008, 2010-2012, 2014-2017 Free
+   Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -13,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 /* First draft by Sergey Poznyakoff */
 
@@ -59,16 +58,17 @@
 #include <mailutils/error.h>
 #include <mailutils/header.h>
 #include <mailutils/message.h>
-#include <mailutils/mutil.h>
+#include <mailutils/util.h>
 #include <mailutils/property.h>
 #include <mailutils/stream.h>
 #include <mailutils/url.h>
 #include <mailutils/observer.h>
 #include <mailutils/errno.h>
 #include <mailutils/locker.h>
-#include <mailbox0.h>
-#include <registrar0.h>
-#include <amd.h>
+#include <mailutils/sys/mailbox.h>
+#include <mailutils/sys/registrar.h>
+#include <mailutils/sys/amd.h>
+#include <mailutils/io.h>
 #include <maildir.h>
 
 #ifndef PATH_MAX 
@@ -85,19 +85,24 @@ struct _maildir_message
 
 
 /* Attribute handling.
-   FIXME: P (Passed), D (Draft) and F (Flagged) are not handled */
+   FIXME: P (Passed) is not handled */
 
 static struct info_map {
   char letter;
   int flag;
 } info_map[] = {
+  { 'D', MU_ATTRIBUTE_DRAFT },
+  { 'F', MU_ATTRIBUTE_FLAGGED },
+  { 'P', 0 }, /* (passed): the user has resent/forwarded/bounced this
+		 message to someone else. */
   { 'R', MU_ATTRIBUTE_READ },
   { 'S', MU_ATTRIBUTE_SEEN },
   { 'T', MU_ATTRIBUTE_DELETED },
+  { 'a', MU_ATTRIBUTE_ANSWERED },
 };
 #define info_map_size (sizeof (info_map) / sizeof (info_map[0]))
 
-/* NOTE: BUF must be at least 7 bytes long */
+/* NOTE: BUF must be at least info_map_size bytes long */
 static int
 flags_to_info (int flags, char *buf)
 {
@@ -165,11 +170,8 @@ maildir_message_cmp (struct _amd_message *a, struct _amd_message *b)
     if ((rc = (*name_a - *name_b)))
       return rc;
 
-  if (*name_a == ':' && *name_b == ':')
-    {
-      name_a++;
-      name_b++;
-    }
+  if ((*name_a == ':' || *name_a == 0) && (*name_b == ':' || *name_b == 0))
+    return 0;
 
   return *name_a - *name_b;
 }
@@ -236,8 +238,9 @@ read_random (void *buf, size_t size)
   return rc != size;
 }
 
-char *
-maildir_mkfilename (const char *directory, const char *suffix, const char *name)
+int
+maildir_mkfilename (const char *directory, const char *suffix, const char *name,
+		    char **ret_name)
 {
   size_t size = strlen (directory) + 1 + strlen (suffix) + 1;
   char *tmp;
@@ -246,22 +249,25 @@ maildir_mkfilename (const char *directory, const char *suffix, const char *name)
     size += 1 + strlen (name);
   
   tmp = malloc (size);
+  if (!tmp)
+    return errno;
   sprintf (tmp, "%s/%s", directory, suffix);
   if (name)
     {
       strcat (tmp, "/");
       strcat (tmp, name);
     }
-  return tmp;
+  *ret_name = tmp;
+  return 0;
 }
 
-static char *
-mk_info_filename (char *directory, char *suffix, char *name, int flags)
+static int
+mk_info_filename (char *directory, char *suffix, char *name, int flags,
+		  char **ret_name)
 {
-  char fbuf[9];
+  char fbuf[info_map_size + 1];
   char *tmp;
   int namelen;
-  size_t size;
 
   tmp = strchr (name, ':');
   if (!tmp)
@@ -269,19 +275,10 @@ mk_info_filename (char *directory, char *suffix, char *name, int flags)
   else
     namelen = tmp - name;
 
-  size = strlen (directory)
-          + 1 + strlen (suffix)
-          + 1 + namelen + 1;
-
   flags_to_info (flags, fbuf);
-  size += 3 + strlen (fbuf);
 
-  tmp = malloc (size);
-  if (!fbuf[0])
-    sprintf (tmp, "%s/%s/%*.*s:2", directory, suffix, namelen, namelen, name);
-  else
-    sprintf (tmp, "%s/%s/%*.*s:2,%s", directory, suffix, namelen, namelen, name, fbuf);
-  return tmp;
+  return mu_asprintf (ret_name, "%s/%s/%*.*s:2,%s",
+		      directory, suffix, namelen, namelen, name, fbuf);
 }
 
 char *
@@ -337,8 +334,7 @@ static int
 maildir_cur_message_name (struct _amd_message *amsg, char **pname)
 {
   struct _maildir_message *msg = (struct _maildir_message *) amsg;
-  *pname = maildir_mkfilename (amsg->amd->name, msg->dir, msg->file_name);
-  return 0;
+  return maildir_mkfilename (amsg->amd->name, msg->dir, msg->file_name, pname);
 }
 
 static int
@@ -352,9 +348,11 @@ maildir_new_message_name (struct _amd_message *amsg, int flags, int expunge,
       *pname = NULL;
     }
   else if (strcmp (msg->dir, CURSUF) == 0)
-    *pname = mk_info_filename (amsg->amd->name, CURSUF, msg->file_name, flags);
+    return mk_info_filename (amsg->amd->name, CURSUF, msg->file_name, flags,
+			     pname);
   else
-    *pname = maildir_mkfilename (amsg->amd->name, msg->dir, msg->file_name);
+    return maildir_mkfilename (amsg->amd->name, msg->dir, msg->file_name,
+			       pname);
   return 0;
 }
 
@@ -390,7 +388,16 @@ static void
 maildir_delete_file (char *dirname, char *filename)
 {
   struct stat st;
-  char *name = maildir_mkfilename (dirname, filename, NULL);
+  char *name;
+  int rc;
+
+  rc = maildir_mkfilename (dirname, filename, NULL, &name);
+  if (rc)
+    {
+      mu_error ("maildir: failed to create file name: %s",
+		mu_strerror (errno));
+      return;
+    }
 
   if (stat (name, &st) == 0)
     {
@@ -405,21 +412,32 @@ maildir_delete_file (char *dirname, char *filename)
 static int
 maildir_opendir (DIR **dir, char *name, int permissions)
 {
+  int rc = 0;
+      
   *dir = opendir (name);
   if (!*dir)
     {
       if (errno == ENOENT)
 	{
 	  if (mkdir (name, permissions))
-	    return errno;
+	    {
+	      rc = errno;
+	      mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
+			("can't create directory %s: %s",
+			 name, mu_strerror (rc)));
+	      return rc;
+	    }
+	  
 	  *dir = opendir (name);
-	  if (!*dir)
-	    return errno;
+	  if (*dir)
+	    return 0;
 	}
-      else
-	return errno;
+      rc = errno;
+      mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
+		("can't open directory %s: %s",
+		 name, mu_strerror (rc)));
     }
-  return 0;
+  return rc;
 }
 
 static int
@@ -431,15 +449,21 @@ maildir_create (struct _amd_data *amd, int flags)
   for (i = 0; i < 3; i++)
     {
       DIR *dir;
-      char *tmpname = maildir_mkfilename (amd->name, dirs[i], NULL);
-      int rc = maildir_opendir (&dir, tmpname,
-				PERMS |
-				mu_stream_flags_to_mode (amd->mailbox->flags,
-							 1));
+      int rc;
+      char *tmpname;
+
+      rc = maildir_mkfilename (amd->name, dirs[i], NULL, &tmpname);
+      if (rc)
+	return rc;
+      
+      rc = maildir_opendir (&dir, tmpname,
+			    PERMS |
+			    mu_stream_flags_to_mode (amd->mailbox->flags,
+						     1));
+      free (tmpname);
       if (rc)
 	return rc;
       closedir (dir);
-      free (tmpname);
     }
   return 0;
 }
@@ -456,10 +480,16 @@ maildir_msg_init (struct _amd_data *amd, struct _amd_message *amm)
   char *name, *fname;
   struct stat st;
   int i;
+  int rc;
   
   name = maildir_uniq (amd, -1);
-  fname = maildir_mkfilename (amd->name, NEWSUF, name);
-
+  rc = maildir_mkfilename (amd->name, NEWSUF, name, &fname);
+  if (rc)
+    {
+      free (name);
+      return rc;
+    }
+  
   msg->dir = TMPSUF;
   
   for (i = 0; i < NTRIES; i++)
@@ -477,7 +507,7 @@ maildir_msg_init (struct _amd_data *amd, struct _amd_message *amm)
     }
   free (fname);
   free (name);
-  return MU_ERR_BAD_RESUMPTION;
+  return EAGAIN;
 }
 
 static int
@@ -485,35 +515,58 @@ maildir_msg_finish_delivery (struct _amd_data *amd, struct _amd_message *amm,
 			     const mu_message_t orig_msg)
 {
   struct _maildir_message *msg = (struct _maildir_message *) amm;
-  char *oldname = maildir_mkfilename (amd->name, TMPSUF, msg->file_name);
-  char *newname;
+  char *oldname, *newname;
   mu_attribute_t attr;
   int flags;
+  int rc;
+  
+  rc = maildir_mkfilename (amd->name, TMPSUF, msg->file_name, &oldname);
+  if (rc)
+    return rc;
   
   if (mu_message_get_attribute (orig_msg, &attr) == 0
       && mu_attribute_is_read (attr)
       && mu_attribute_get_flags (attr, &flags) == 0)
     {
       msg->dir = CURSUF;
-      newname = mk_info_filename (amd->name, CURSUF, msg->file_name, flags);
+      rc = mk_info_filename (amd->name, CURSUF, msg->file_name, flags,
+			     &newname);
     }
   else
     {
       msg->dir = NEWSUF;
-      newname = maildir_mkfilename (amd->name, NEWSUF, msg->file_name);
-    }
-    
-  unlink (newname);
-  if (link (oldname, newname) == 0)
-    unlink (oldname);
-  else
-    {
-      return errno; /* FIXME? */
+      rc = maildir_mkfilename (amd->name, NEWSUF, msg->file_name, &newname);
     }
 
+  if (rc == 0)
+    {
+      if (unlink (newname) && errno != ENOENT)
+	{
+	  rc = errno;
+	  mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
+		    ("can't unlink %s: %s",
+		     newname, mu_strerror (errno)));
+	}
+      else if (link (oldname, newname) == 0)
+	{
+	  if (unlink (oldname))
+	    mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
+		      ("can't unlink %s: %s",
+		       oldname, mu_strerror (errno)));
+	}
+      else
+	{
+	  rc = errno;
+	  mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
+		    ("renaming %s to %s failed: %s",
+		     oldname, newname, mu_strerror (rc)));
+	}
+
+    }
+  
   free (oldname);
   free (newname);
-  return 0;
+  return rc;
 }
 
 
@@ -525,7 +578,14 @@ maildir_flush (struct _amd_data *amd)
   int rc;
   DIR *dir;
   struct dirent *entry;
-  char *tmpname = maildir_mkfilename (amd->name, TMPSUF, NULL);
+  char *tmpname;
+
+  if (!(amd->mailbox->flags & MU_STREAM_WRITE))
+    return EACCES;
+
+  rc = maildir_mkfilename (amd->name, TMPSUF, NULL, &tmpname);
+  if (rc)
+    return rc;
 
   rc = maildir_opendir (&dir, tmpname,
 			PERMS |
@@ -559,95 +619,119 @@ int
 maildir_deliver_new (struct _amd_data *amd, DIR *dir)
 {
   struct dirent *entry;
+  int err = 0;
 
+  if (!(amd->mailbox->flags & MU_STREAM_WRITE))
+    return EACCES;
   while ((entry = readdir (dir)))
     {
       char *oldname, *newname;
-	
+      int rc;
+      
       switch (entry->d_name[0])
 	{
 	case '.':
 	  break;
 
 	default:
-	  oldname = maildir_mkfilename (amd->name, NEWSUF, entry->d_name);
-	  newname = mk_info_filename (amd->name, CURSUF, entry->d_name, 0);
-	  rename (oldname, newname);
+	  rc = maildir_mkfilename (amd->name, NEWSUF, entry->d_name, &oldname);
+	  if (rc)
+	    return rc;
+	  rc = mk_info_filename (amd->name, CURSUF, entry->d_name, 0, &newname);
+	  if (rc)
+	    {
+	      free (oldname);
+	      return rc;
+	    }
+	  if (rename (oldname, newname))
+	    {
+	      err = MU_ERR_FAILURE;
+	      mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
+			("renaming %s to %s failed: %s",
+			 oldname, newname, mu_strerror (errno)));
+	    }
 	  free (oldname);
 	  free (newname);
 	}
     }
-  return 0;
-}
-
-static struct _maildir_message *
-maildir_message_lookup (struct _amd_data *amd, char *file_name)
-{
-  struct _maildir_message *msg;
-  char *p = strchr (file_name, ':');
-  size_t length = p ? p - file_name : strlen (file_name);
-  size_t i;
-  
-  /*FIXME: Replace linear search with binary one! */
-  for (i = 0; i < amd->msg_count; i++)
-    {
-      msg = (struct _maildir_message *) amd->msg_array[i];
-
-      if (strlen (msg->file_name) <= length
-	  && memcmp (msg->file_name, file_name, length) == 0)
-	return msg;
-    }
-  return NULL;
+  return err;
 }
 
 static int
 maildir_scan_dir (struct _amd_data *amd, DIR *dir, char *dirname)
 {
   struct dirent *entry;
-  struct _maildir_message *msg;
+  struct _maildir_message *msg, key;
   char *p;
-  int insert;
-
+  size_t index;
+  int rc = 0;
+  int need_sort = 0;
+  struct stat st;
+  
   while ((entry = readdir (dir)))
     {
-      switch (entry->d_name[0])
+      char *fname;
+      
+      if (entry->d_name[0] == '.')
+	continue;
+
+      rc = maildir_mkfilename (amd->name, dirname, entry->d_name, &fname);
+      if (rc)
 	{
-	case '.':
-	  break;
-
-	default:
-	  msg = maildir_message_lookup (amd, entry->d_name);
-	  if (msg)
-	    {
-	      free (msg->file_name);
-	      insert = 0;
-	    }
-	  else
-	    {
-	      msg = calloc (1, sizeof(*msg));
-	      insert = 1;
-	    }
-	  
-	  msg->dir = dirname;
-	  msg->file_name = strdup (entry->d_name);
-
-	  p = maildir_name_info_ptr (msg->file_name);
-	  if (p)
-	    msg->amd_message.attr_flags = info_to_flags (p);
-	  else
-	    msg->amd_message.attr_flags = 0;
-	  msg->amd_message.orig_flags = msg->amd_message.attr_flags;
-	  if (insert)
-	    _amd_message_insert (amd, (struct _amd_message*) msg);
+	  mu_diag_funcall (MU_DIAG_ERROR, "maildir_mkfilename",
+			   entry->d_name, rc);
+	  continue;
 	}
+      
+      if (stat (fname, &st))
+	{
+	  rc = errno;
+	  mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
+		    ("can't stat %s: %s", fname, mu_strerror (rc)));
+	  free (fname);
+	  continue;
+	}
+
+      free (fname);
+
+      if (!S_ISREG (st.st_mode))
+	continue;
+      
+      msg = calloc (1, sizeof (*msg));
+      if (!msg)
+	{
+	  rc = ENOMEM;
+	  break;
+	}
+      key.file_name = entry->d_name;
+      if (!amd_msg_lookup (amd, (struct _amd_message *) &key, &index))
+	continue;
+      rc = _amd_message_append (amd, (struct _amd_message *) msg);
+      if (rc)
+	{
+	  free (msg);
+	  break;
+	}
+	      
+      msg->dir = dirname;
+      msg->file_name = strdup (entry->d_name);
+
+      p = maildir_name_info_ptr (msg->file_name);
+      if (p)
+	msg->amd_message.attr_flags = info_to_flags (p);
+      else
+	msg->amd_message.attr_flags = 0;
+      msg->amd_message.orig_flags = msg->amd_message.attr_flags;
+      need_sort = 1;
     }
-  return 0;
+
+  if (rc == 0 && need_sort)
+    amd_sort (amd);
+  return rc;
 }
 
 static int
-maildir_scan0 (mu_mailbox_t mailbox, size_t msgno MU_ARG_UNUSED,
-	       size_t *pcount, 
-	       int do_notify)
+maildir_scan_unlocked (mu_mailbox_t mailbox, size_t *pcount, int do_notify)
 {
   struct _amd_data *amd = mailbox->data;
   DIR *dir;
@@ -656,30 +740,30 @@ maildir_scan0 (mu_mailbox_t mailbox, size_t msgno MU_ARG_UNUSED,
   struct stat st;
   size_t i;
   
-  if (amd == NULL)
-    return EINVAL;
-  if (mailbox->flags & MU_STREAM_APPEND)
-    return 0;
-  mu_monitor_wrlock (mailbox->monitor);
-
   /* 1st phase: Flush tmp/ */
   maildir_flush (amd);
 
   /* 2nd phase: Scan and deliver messages from new */
-  name = maildir_mkfilename (amd->name, NEWSUF, NULL);
-
+  status = maildir_mkfilename (amd->name, NEWSUF, NULL, &name);
+  if (status)
+    return status;
+  
   status = maildir_opendir (&dir, name,
 			    PERMS |
 			    mu_stream_flags_to_mode (mailbox->flags, 1));
   if (status == 0)
     {
-      maildir_deliver_new (amd, dir);
+      if (maildir_deliver_new (amd, dir))
+	status = maildir_scan_dir (amd, dir, NEWSUF);
       closedir (dir);
     }
   free (name);
 
-  name = maildir_mkfilename (amd->name, CURSUF, NULL);
   /* 3rd phase: Scan cur/ */
+  status = maildir_mkfilename (amd->name, CURSUF, NULL, &name);
+  if (status)
+    return status;
+  
   status = maildir_opendir (&dir, name,
 			    PERMS |
 			    mu_stream_flags_to_mode (mailbox->flags, 1));
@@ -705,28 +789,120 @@ maildir_scan0 (mu_mailbox_t mailbox, size_t msgno MU_ARG_UNUSED,
   if (pcount)
     *pcount = amd->msg_count;
 
-  /* Reset the uidvalidity.  */
-  if (amd->msg_count > 0)
-    {
-      if (amd->uidvalidity == 0)
-	{
-	  amd->uidvalidity = (unsigned long) time (NULL);
-	  /* FIXME amd->uidnext = amd->msg_count + 1;*/
-	  /* Tell that we have been modified for expunging.  */
-	  if (amd->msg_count)
-	    {
-	      amd_message_stream_open (amd->msg_array[0]);
-	      amd_message_stream_close (amd->msg_array[0]);
-	      amd->msg_array[0]->attr_flags |= MU_ATTRIBUTE_MODIFIED;
-	    }
-	}
-    }
-
-  /* Clean up the things */
-  amd_cleanup (mailbox);
   return status;
 }
 
+static int
+maildir_scan0 (mu_mailbox_t mailbox, size_t msgno MU_ARG_UNUSED,
+	       size_t *pcount, 
+	       int do_notify)
+{
+  struct _amd_data *amd = mailbox->data;
+  int rc;
+  
+  if (amd == NULL)
+    return EINVAL;
+  if (mailbox->flags & MU_STREAM_APPEND)
+    return 0;
+  mu_monitor_wrlock (mailbox->monitor);
+  rc = maildir_scan_unlocked (mailbox, pcount, do_notify);
+  mu_monitor_unlock (mailbox->monitor);
+  return rc;
+}
+  
+static int
+maildir_size_dir (struct _amd_data *amd, char *dirsuf, mu_off_t *psize)
+{
+  DIR *dir;
+  struct dirent *entry;
+  int rc = 0;
+  struct stat st;
+  char *name;
+  
+  rc = maildir_mkfilename (amd->name, dirsuf, NULL, &name);
+  if (rc)
+    return rc;
+  dir = opendir (name);
+  
+  if (!dir)
+    {
+      rc = errno;
+      mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
+		("can't open directory %s: %s", name, mu_strerror (rc)));
+      free (name);
+      if (rc == ENOENT)
+	return 0;
+      return rc;
+    }
+
+  while ((entry = readdir (dir)))
+    {
+      char *fname;
+      
+      if (entry->d_name[0] == '.')
+	continue;
+
+      rc = maildir_mkfilename (amd->name, dirsuf, entry->d_name, &fname);
+      if (rc)
+	{
+	  mu_diag_funcall (MU_DIAG_ERROR, "maildir_mkfilename",
+			   entry->d_name, rc);
+	  continue;
+	}
+      
+      if (stat (fname, &st))
+	{
+	  rc = errno;
+	  mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
+		    ("can't stat %s: %s", fname, mu_strerror (rc)));
+	  free (fname);
+	  continue;
+	}
+
+      free (fname);
+
+      if (S_ISREG (st.st_mode))
+	*psize += st.st_size;
+    }
+
+  closedir (dir);
+  free (name);
+  
+  return 0;
+}
+
+static int
+maildir_size_unlocked (struct _amd_data *amd, mu_off_t *psize)
+{
+  mu_off_t size = 0;
+  int rc;
+
+  rc = maildir_size_dir (amd, NEWSUF, &size);
+  if (rc)
+    return rc;
+  rc = maildir_size_dir (amd, CURSUF, &size);
+  if (rc)
+    return rc;
+  *psize = size;
+  return 0;
+}
+
+static int
+maildir_size (mu_mailbox_t mailbox, mu_off_t *psize)
+{
+  struct _amd_data *amd = mailbox->data;
+  int rc;
+  
+  if (amd == NULL)
+    return EINVAL;
+
+  mu_monitor_wrlock (mailbox->monitor);
+  rc = maildir_size_unlocked (amd, psize);
+  mu_monitor_unlock (mailbox->monitor);
+
+  return rc;
+}
+  
 
 static int
 maildir_qfetch (struct _amd_data *amd, mu_message_qid_t qid)
@@ -765,6 +941,85 @@ maildir_qfetch (struct _amd_data *amd, mu_message_qid_t qid)
   return 0;
 }
 
+
+static int
+maildir_remove (struct _amd_data *amd)
+{
+  int i;
+  static char *suf[3] = { NEWSUF, CURSUF, TMPSUF };
+  int rc = 0;
+
+  for (i = 0; rc == 0 && i < 3; i++)
+    {
+      char *name;
+
+      rc = maildir_mkfilename (amd->name, suf[i], NULL, &name);
+      if (rc)
+	return rc;
+      rc = amd_remove_dir (name);
+      if (rc)
+	mu_diag_output (MU_DIAG_WARNING,
+			"removing contents of %s failed: %s", name,
+			mu_strerror (rc));
+      free (name);
+    }
+  
+  return rc;
+}
+
+
+static int
+maildir_chattr_msg (struct _amd_message *amsg, int expunge)
+{
+  struct _maildir_message *mp = (struct _maildir_message *) amsg;
+  struct _amd_data *amd = amsg->amd;
+  int rc;
+  char *new_name;
+  
+  rc = amd->new_msg_file_name (amsg, amsg->attr_flags, expunge, &new_name);
+  if (rc)
+    return rc;
+  if (!new_name)
+    {
+      if (unlink (mp->file_name))
+	{
+	  rc = errno;
+	  mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
+		    ("can't unlink %s: %s",
+		     mp->file_name, mu_strerror (rc)));
+	}
+    }
+  else
+    {
+      char *cur_name;
+      
+      rc = maildir_cur_message_name (amsg, &cur_name);
+      if (rc)
+	{
+	  free (new_name);
+	  return rc;
+	}
+      if (rename (cur_name, new_name))
+	{
+	  if (errno == ENOENT)
+	    mu_observable_notify (amd->mailbox->observable,
+				  MU_EVT_MAILBOX_CORRUPT,
+				  amd->mailbox);
+	  else
+	    {
+	      rc = errno;
+	      mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_ERROR,
+			("renaming %s to %s failed: %s",
+			 cur_name, new_name, mu_strerror (rc)));
+	    }
+	}
+      free (cur_name);
+    }
+
+  free (new_name);
+  return rc;
+}
+
      
 int
 _mailbox_maildir_init (mu_mailbox_t mailbox)
@@ -788,6 +1043,10 @@ _mailbox_maildir_init (mu_mailbox_t mailbox)
   amd->msg_cmp = maildir_message_cmp;
   amd->message_uid = maildir_message_uid;
   amd->next_uid = maildir_next_uid;
+  amd->remove = maildir_remove;
+  amd->chattr_msg = maildir_chattr_msg;
+  amd->capabilities = MU_AMD_STATUS;
+  amd->mailbox_size = maildir_size;
   
   /* Set our properties.  */
   {
