@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2002, 2005, 2007-2012, 2014-2015 Free Software
+   Copyright (C) 1999-2002, 2005, 2007-2012, 2014-2017 Free Software
    Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -225,7 +225,7 @@ deliver_to_mailbox (mu_mailbox_t mbox, mu_message_t msg,
     }
 #endif
   
-  if (!failed && switch_user_id (auth, 1) == 0)
+  if (!failed)
     {
       status = mu_mailbox_append_message (mbox, msg);
       if (status)
@@ -244,7 +244,6 @@ deliver_to_mailbox (mu_mailbox_t mbox, mu_message_t msg,
 	      failed++;
 	    }
 	}
-      switch_user_id (auth, 0);
     }
 
   mu_mailbox_close (mbox);
@@ -271,6 +270,8 @@ do_delivery (mu_url_t url, mu_message_t msg, const char *name, char **errp)
   mu_mailbox_t mbox;
   int status;
 
+  mu_set_user_email_domain (default_domain);
+  
   if (name && !is_remote_url (url))
     {
       auth = mu_get_auth_by_name (name);
@@ -283,6 +284,10 @@ do_delivery (mu_url_t url, mu_message_t msg, const char *name, char **errp)
 	  return EX_NOUSER;
 	}
 
+      status = mu_set_user_email (name);
+      if (status)
+	mu_error (_("%s: invalid email: %s"), name, mu_strerror (status));
+      
       if (current_uid)
 	auth->change_uid = 0;
 
@@ -314,6 +319,8 @@ do_delivery (mu_url_t url, mu_message_t msg, const char *name, char **errp)
 	    return exit_code = EX_TEMPFAIL;
 	  }
     }
+  else
+    mu_set_user_email (NULL);
   
   if (!url)
     {
