@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2002, 2005-2007, 2009-2012, 2014-2016 Free
+   Copyright (C) 1999-2002, 2005-2007, 2009-2012, 2014-2017 Free
    Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -173,7 +173,8 @@ extern int interactive;
 extern const char *program_version;
 extern char *default_encoding;
 extern char *default_content_type;
-
+extern int skip_empty_attachments;
+  
 /* Functions */
 extern int mail_alias (int argc, char **argv);
 extern int mail_alt (int argc, char **argv);	/* command alternates */
@@ -193,7 +194,7 @@ extern int mail_folders (int argc, char **argv);
 extern int mail_followup (int argc, char **argv);
 extern int mail_from (int argc, char **argv);
 extern int mail_from0 (msgset_t *mspec, mu_message_t msg, void *data);
-extern void mail_compile_headline (struct mailvar_variable *var);
+extern void mail_compile_headline (char const *str);
 
 extern int mail_headers (int argc, char **argv);
 extern int mail_hold (int argc, char **argv);
@@ -258,7 +259,11 @@ extern char *mail_expand_name (const char *name);
 
 extern void send_append_header (char const *text);
 extern void send_append_header2 (char const *name, char const *value, int mode);
-extern int send_attach_file_default (const char *name);
+extern int send_attach_file (int fd,
+			     const char *filename,
+			     const char *content_filename,
+			     const char *content_name,
+			     const char *content_type, const char *encoding);
 
 extern int escape_check_args (int argc, char **argv, int minargs, int maxargs);
 
@@ -288,6 +293,15 @@ extern int escape_attach (int argc, char **argv, compose_env_t *env);
 extern int escape_remove_attachment (int argc, char **argv,
 				     compose_env_t *env);
 
+enum
+  {
+    parse_headers_ok,
+    parse_headers_error,
+    parse_headers_fatal
+  };
+
+extern int parse_headers (mu_stream_t input, compose_env_t *env);
+
 /* Cursor */
 extern void set_cursor (unsigned value);
 extern size_t get_cursor (void);
