@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2003, 2005-2012, 2014-2016 Free Software
+   Copyright (C) 1999-2003, 2005-2012, 2014-2017 Free Software
    Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -40,6 +40,7 @@ static const char *append_folder;
 static const char *move_to_mailbox;
 static const char *script_file;
 static const char *script_lang;
+static char const *script_env[] = { "location=MUA", "phase=post", NULL };
 
 static void
 add_file (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
@@ -390,7 +391,8 @@ main (int argc, char **argv)
 	      exit (1);
 	    }
 	}
-      rc = mu_script_init (incdat.handler, script_file, &incdat.descr);
+      rc = mu_script_init (incdat.handler, script_file, script_env,
+			   &incdat.descr);
       if (rc)
 	{
 	  mu_error (_("script initialization failed: %s"),
