@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005-2007, 2009-2012, 2014-2016 Free Software
+   Copyright (C) 2003, 2005-2007, 2009-2012, 2014-2017 Free Software
    Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -425,7 +425,13 @@ quit (struct mh_whatnow_env *wh, int argc, char **argv, int *status)
 	{
 	  mu_printf (_("draft left on \"%s\"."), wh->draftfile);
 	  if (strcmp (wh->file, wh->draftfile))
-	    rename (wh->file, wh->draftfile);
+	    {
+	      int rc;
+	      rc = mu_rename_file (wh->file, wh->draftfile, MU_COPY_OVERWRITE);
+	      if (rc)
+		mu_error (_("can't rename %s to %s: %s"),
+			  wh->file, wh->draftfile, mu_strerror (rc));
+	    }
 	}
     }
   mu_printf ("\n");
