@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005-2012, 2014-2016 Free Software Foundation,
+   Copyright (C) 2003, 2005-2012, 2014-2017 Free Software Foundation,
    Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -286,7 +286,7 @@ compare_action (void *item, void *data)
   free (a);
   free (b);
 
-  return dp->r; /* go on until the difference is found */
+  return dp->r ? MU_ERR_USER0 : 0; /* go on until the difference is found */
 }
 
 static int
